#ifndef _GROK_MATCHCONF_MACRO_
#define _GROK_MATCHCONF_MACRO_

enum matchconf_macro_type {
  VALUE_LINE,
  VALUE_MATCH,
  VALUE_JSON_COMPLEX,
  VALUE_JSON_SIMPLE,
  VALUE_START,
  VALUE_END,
  VALUE_LENGTH,
};

#ifndef _GPERF_
struct strmacro {
  const char *str;
  int code;
};
#endif

/* this function is generated by gperf */
const struct strmacro *patname2macro(const char *str, unsigned int len);

#endif /* _GROK_MATCHCONF_MACRO_ */
