/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _GROK_CAPTURE_XDR_H_RPCGEN
#define _GROK_CAPTURE_XDR_H_RPCGEN

#include <rpc/rpc.h>


#ifdef __cplusplus
extern "C" {
#endif


struct grok_capture {
	int name_len;
	char *name;
	int subname_len;
	char *subname;
	int pattern_len;
	char *pattern;
	int id;
	int pcre_capture_number;
	int predicate_lib_len;
	char *predicate_lib;
	int predicate_func_name_len;
	char *predicate_func_name;
	struct {
		u_int extra_len;
		char *extra_val;
	} extra;
};
typedef struct grok_capture grok_capture;

/* the xdr functions */

#if defined(__STDC__) || defined(__cplusplus)
extern  bool_t xdr_grok_capture (XDR *, grok_capture*);

#else /* K&R C */
extern bool_t xdr_grok_capture ();

#endif /* K&R C */

#ifdef __cplusplus
}
#endif

#endif /* !_GROK_CAPTURE_XDR_H_RPCGEN */
