/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include "grok_capture.h"

bool_t
xdr_grok_capture (XDR *xdrs, grok_capture *objp)
{
	register int32_t *buf;


	if (xdrs->x_op == XDR_ENCODE) {
		 if (!xdr_int (xdrs, &objp->name_len))
			 return FALSE;
		 if (!xdr_string (xdrs, &objp->name, ~0))
			 return FALSE;
		 if (!xdr_int (xdrs, &objp->subname_len))
			 return FALSE;
		 if (!xdr_string (xdrs, &objp->subname, ~0))
			 return FALSE;
		 if (!xdr_int (xdrs, &objp->pattern_len))
			 return FALSE;
		 if (!xdr_string (xdrs, &objp->pattern, ~0))
			 return FALSE;
		buf = XDR_INLINE (xdrs, 3 * BYTES_PER_XDR_UNIT);
		if (buf == NULL) {
			 if (!xdr_int (xdrs, &objp->id))
				 return FALSE;
			 if (!xdr_int (xdrs, &objp->pcre_capture_number))
				 return FALSE;
			 if (!xdr_int (xdrs, &objp->predicate_lib_len))
				 return FALSE;

		} else {
		IXDR_PUT_LONG(buf, objp->id);
		IXDR_PUT_LONG(buf, objp->pcre_capture_number);
		IXDR_PUT_LONG(buf, objp->predicate_lib_len);
		}
		 if (!xdr_string (xdrs, &objp->predicate_lib, ~0))
			 return FALSE;
		 if (!xdr_int (xdrs, &objp->predicate_func_name_len))
			 return FALSE;
		 if (!xdr_string (xdrs, &objp->predicate_func_name, ~0))
			 return FALSE;
		 if (!xdr_bytes (xdrs, (char **)&objp->extra.extra_val, (u_int *) &objp->extra.extra_len, ~0))
			 return FALSE;
		return TRUE;
	} else if (xdrs->x_op == XDR_DECODE) {
		 if (!xdr_int (xdrs, &objp->name_len))
			 return FALSE;
		 if (!xdr_string (xdrs, &objp->name, ~0))
			 return FALSE;
		 if (!xdr_int (xdrs, &objp->subname_len))
			 return FALSE;
		 if (!xdr_string (xdrs, &objp->subname, ~0))
			 return FALSE;
		 if (!xdr_int (xdrs, &objp->pattern_len))
			 return FALSE;
		 if (!xdr_string (xdrs, &objp->pattern, ~0))
			 return FALSE;
		buf = XDR_INLINE (xdrs, 3 * BYTES_PER_XDR_UNIT);
		if (buf == NULL) {
			 if (!xdr_int (xdrs, &objp->id))
				 return FALSE;
			 if (!xdr_int (xdrs, &objp->pcre_capture_number))
				 return FALSE;
			 if (!xdr_int (xdrs, &objp->predicate_lib_len))
				 return FALSE;

		} else {
		objp->id = IXDR_GET_LONG(buf);
		objp->pcre_capture_number = IXDR_GET_LONG(buf);
		objp->predicate_lib_len = IXDR_GET_LONG(buf);
		}
		 if (!xdr_string (xdrs, &objp->predicate_lib, ~0))
			 return FALSE;
		 if (!xdr_int (xdrs, &objp->predicate_func_name_len))
			 return FALSE;
		 if (!xdr_string (xdrs, &objp->predicate_func_name, ~0))
			 return FALSE;
		 if (!xdr_bytes (xdrs, (char **)&objp->extra.extra_val, (u_int *) &objp->extra.extra_len, ~0))
			 return FALSE;
	 return TRUE;
	}

	 if (!xdr_int (xdrs, &objp->name_len))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->name, ~0))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->subname_len))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->subname, ~0))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->pattern_len))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->pattern, ~0))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->id))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->pcre_capture_number))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->predicate_lib_len))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->predicate_lib, ~0))
		 return FALSE;
	 if (!xdr_int (xdrs, &objp->predicate_func_name_len))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->predicate_func_name, ~0))
		 return FALSE;
	 if (!xdr_bytes (xdrs, (char **)&objp->extra.extra_val, (u_int *) &objp->extra.extra_len, ~0))
		 return FALSE;
	return TRUE;
}
